@@ -2,11 +2,11 @@ from elf_to_shellcode.lib.shellcode import Shellcode, create_make_shellcode
 
 
 class ArmX32Shellcode(Shellcode):
-    def __init__(self, elffile, shellcode_data, endian, **kwargs):
+    def __init__(self, elffile, shellcode_data, args, **kwargs):
         super(ArmX32Shellcode, self).__init__(
             elffile=elffile,
             shellcode_data=shellcode_data,
-            endian=endian,
+            args=args,
             arch="arm32",
             mini_loader_little_endian="mini_loader_arm_x32{}.shellcode",
             mini_loader_big_endian="mini_loader_arm_x32be{}.shellcode",
